function [ b, res, msft ] = l1rgres(y, X, verb, alg)
% l1rgres	minimum l_1 misfit regression using linear programming
%
%	WARNING: Matlab's lp implementation is unstable!
%
% Usage:
%        [ b, res, msft ] = l1rgres(y, X, verb, alg)
%
% Arguments:
%        y	n-vector of data
%        X      nxp design matrix
%        verb	if verb = -1, LP runs silently.
%	 alg	string for the linear programming algorithm to use.
%		If alg == 'lp', uses the MATLAB Optimization toolbox
%		LP.M routine.
%		Defaults to 'lp'.
%
%
% Results:
%        b	p-vector of fitted coefficients
%        res	n-vector of final residuals
%        msft	l_1 misfit of solution
%
% P.B. Stark 	stark@stat.berkeley.edu
% 10 July 1997


%
% check dimensions
[n,p] = size(X);
[ny, py] = size(y);

if (n ~= ny),
	error('number of rows of y ~= number of rows of X')
end

if (py ~= 1),
	error('y has more than one column')
end

% output level?
talk = 0;
if (nargin >= 3),
	talk = verb;
end;

%
% set up slack variables and their bounds
% 
% want 	to minimize 
%	sum_{j=1}^n ( pos_j + neg_j )
% subject to
%	y = Ab + pos - neg
%	0 <= pos <= infty
%	0 <= neg <= infty
%
% what to do depends on the linear programming algorithm used.
%
lpalg = 'lp';
if (nargin == 4),
	lpalg = alg;
	if (~strcmp(lpalg,'lp') ),
		error(['1: the algorithm ' lpalg ' is not supported'])
	end;
end;

%
% use ols to get a cheap feasible starting model
[xols, olsres, rssols] = wls(y, X, ones(n,1));

% the residuals give the values of the slacks
pos = zeros(size(olsres));
neg = pos;
pos(olsres > 0) = olsres(olsres > 0);
neg(olsres < 0) = -olsres(olsres < 0);
clear olsres;				% free the memory

% now the lp phase
if (talk >= 0),
	disp('starting LP phase')
end

% cases, by LP algorithm invoked
%
if (strcmp(lpalg,'lp')),
	lpn = p + 2*n;			% number of variables, including slacks
	lpm = n;			% total number of constraints
	neq = n;			% number of equality constraints
	f = [zeros(1,p) ones(1,2*n)];	% objective vector (a row vector)
	vlb = [-inf * ones(p,1); ...
       		zeros(2*n,1) ];		% lower bounds on variables
	vub = inf * ones(lpn,1);	% upper bounds
	A = [X ones(n) -ones(n)];	% add the slacks to the constraints
					% this gets HUGE quickly!
	x0 = [xols; pos; neg];
	lpx = lp(f, A, y, vlb, vub, x0, neq, talk);
	msft = f * lpx;
	b = lpx(1:p);			% extract solution
	res = y - X*b;
else
	error(['impossible error: algorithm ', alg, ' not supported'])
end;

return