function [ b, res, se, it ] = rwlsBoot(y,X,XP,nboot,wfunc,param,start)
% rwlsBoot iteratively re-weighted least squares
%          Can solve "robust" regression problems using
%          robust weights.
%	   uses the bootstrap to estimate the SE of XP*b.
% Usage:
%        [b, res, se, it ] = rwlsBoot(y,X,XP,nboot,wfunc,param,start)
%
% Arguments:
%        y	n-vector of data
%        X      nxp design matrix
%	 XP	matrix to apply to b to get the vector of predictions
%		whose SE is sought (by bootstrap)
%	 nboot	number of bootstrap samples to draw
%        wfunc	the name of the weight function subroutine, passed as a string
%               W = wfunc(res, scale, param) must take three arguments:
%			res 	vector of residuals
%			scale	resistant estimate of scale (MAD/.6745)
%			param	array of particular parameters passed by caller
%		wfunc must return a vector W of the same dimension as res
%        param	a vector of parameters to pass to wfunc
%	 start 	the method used to find the estimate of scale---see the
%	 	documentation of irwls.m
%
% Results:
%        b	p-vector of fitted coefficients
%        res	n-vector of final residuals
%        se	vector of bootstrap standard errors of XP*b
%	 it	total number of iterations (initial + bootstrap)

% P.B. Stark 	stark@stat.berkeley.edu
% 11 July 1997

chgTol = 0.001;					% tolerance for convergence 
						% of coefficients

[ b, res, wssr, it ] = irwls(y,X,wfunc,param,start);

% bootstrap the uncertainties

[nd,p] = size(X);
yhat = X*b;
bb = zeros(p,nboot);
for i = 1:nboot,
	resam = yhat + res(ceil(nd*rand(nd,1)));
	[bb(:,i), bres, bwssr, adit] = irwls(resam,X,wfunc,param,start);
	it = it + adit;
end

if (nboot > 1),
	smat = XP*bb;
	se = std(smat')';
else
	se = [];
end
return