function [b, res, wssr] = wls(y,X,W)
%  wls 	Multiple linear regression using weighted least squares.
%   
%  Usage:
%	[b, res, wssr] = wls(y,X,W) returns the p-vector b of 
%		regression coefficients such that 
%			(Xb - y)' W'W (Xb - y)    (**)
%		is minimal, along with the residual n-vector 
%			res = y - Xb, 
%		and the weighted sum of squared residuals 
%			wssr = (Xb - y)'  W  (Xb - y).
%  Arguments:
%	y	n-vector of observations
%	X	nxp design matrix
%	W	nxn positive semi-definite square matrix of weights,
%		or a vector of nonnegative weights.	
%		Note that W is the square-root of what one would think
%		of as the data error covariance matrix.
%
%
%  Algorithm:	Uses the QR decomposition as follows.
%		Define Z = WX and d = Wy. Then (**) can
%		be written as an OLS (ordinary least squares)
%		problem: minimize
%			|| Zb - d ||^2.
%		If [Q, R] is the QR decomposition of Z,
%		stationarity yields
%			2Z'(Zb - d) = 0,
%		or 	Z'Zb = Z'd.
%		The QR decomposition gives
%			R'Rb = R'Q'd; for nonsingular R,
%			Rb = Q'd.
%		Matlab can solve this triangular system stably using
%			b = R\(Q'd).
%

if  nargin < 3,              
    error('wls requires three input arguments.');      
end 

% Check that X, y, and W have compatible dimensions
[n,p] = size(X);
[ny,py] = size(y);
[nW,pW] = size(W);
dimW = max(nW, pW);
minDimW = min(nW, pW);
%
if (n ~= ny), 
	error( 'The number of rows in y must equal the number of rows in X' ); 
elsif ( py ~= 1 ), 
	error( 'y must be a vector, not a matrix' ); 
elsif ( minDimW > 1 & nW ~= pW ),
	error( 'W must be a square matrix or a vector' );
elsif ( dimW ~= n ),
	error( 'W must have the same number of rows as X and y' );
elsif ( minDimW == 1 & min(W) < 0 ),
	error( 'W must be nonnegative' );
end

% build the weighted design matrix and data

if ( minDimW == 1),
	W = diag(W);		% in case W is a vector
end

Z = W*X;
d = W*y;

[Q, R] = qr(Z, 0);
b = R\(Q'*d);

yHat = X*b;			% predicted data values
res = y - yHat;			% residuals
wssr = norm(W*res);		% weighted sum of squared residuals

return;

